/**
 * Copyright (C) 2010-2012 enStratus Networks Inc
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.terremark;

import static org.jclouds.concurrent.MoreExecutors.sameThreadExecutor;

import org.apache.log4j.Logger;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.CloudException;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.jclouds.terremark.compute.TerremarkComputeServices;
import org.dasein.cloud.jclouds.terremark.identity.TerremarkIdentityServices;
import org.dasein.cloud.jclouds.terremark.network.TerremarkNetworkServices;
import org.jclouds.compute.ComputeServiceContextFactory;
import org.jclouds.concurrent.config.ExecutorServiceModule;
import org.jclouds.logging.log4j.config.Log4JLoggingModule;
import org.jclouds.rest.RestContext;
import org.jclouds.trmk.vcloudexpress.TerremarkVCloudExpressAsyncClient;
import org.jclouds.trmk.vcloudexpress.TerremarkVCloudExpressClient;

import com.google.common.collect.ImmutableSet;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class Terremark extends AbstractCloud {
    static private final Logger logger = Logger.getLogger(Terremark.class);
    
    static public @Nonnull String toId(@Nonnull String id) {
        int idx = id.lastIndexOf("/");
        
        if( idx == -1 ) {
            return id;
        }
        return id.substring(idx+1);
    }
    
    public Terremark() { }
    
    public @Nonnull RestContext<TerremarkVCloudExpressClient,TerremarkVCloudExpressAsyncClient> getCloudClient() throws CloudException {
        ProviderContext ctx = getContext();
        
        if( ctx == null ) {
            throw new CloudException("No context was set for this request");
        }
        String user = new String(ctx.getAccessPublic());
        String key = new String(ctx.getAccessPrivate());
        
        return new ComputeServiceContextFactory().createContext("trmk-vcloudexpress", user, key, ImmutableSet.of(new Log4JLoggingModule(), new ExecutorServiceModule(sameThreadExecutor(), sameThreadExecutor()))).getProviderSpecificContext();
    }

    @Override
    public @Nonnull String getCloudName() {
        return "Terremark";
    }

    @Override
    public @Nonnull TerremarkComputeServices getComputeServices() {
        return new TerremarkComputeServices(this);
    }
    
    @Override
    public @Nonnull Vdc getDataCenterServices() {
        return new Vdc(this);
    }

    @Override
    public @Nonnull TerremarkNetworkServices getNetworkServices() {
        return new TerremarkNetworkServices(this);
    }
    
	@Override
	public TerremarkIdentityServices getIdentityServices() {
	    return new TerremarkIdentityServices(this);
	}
    
    @Override
    public @Nonnull String getProviderName() {
        return "Terremark";
    }
    
    @Override
    public @Nullable String testContext() {
        try {
            ProviderContext ctx = getContext();

            if( ctx == null ) {
                return null;
            }
            if( !getComputeServices().getVirtualMachineSupport().isSubscribed() ) {
                return null;
            }
            return ctx.getAccountNumber();
        }
        catch( Throwable t ) {
            logger.warn("Failed to test Terremark connection context: " + t.getMessage());
            if( logger.isDebugEnabled() ) {
                t.printStackTrace();
            }
            return null;
        }
    }
}
